:li

!!! Listing "Spezialisierung"

// Klassendefinitionen

#include <stream.h>
#include <cxxcls/pobject.h>
#include <cxxcls/pstring.h>
#include <cxxcls/vset.h>


class PEmployee : public PVirtual {
    protected:
        PString  Name;
        o_u4b    Salary;
    public:
        PEmployee( char *aName, o_u4b aSalary );
        virtual o_u4b hash() const;
        virtual o_4b  compare( const PVirtual & ) const;
        void          SetName( char *aNewName );
        void          SetSalary( o_u4b aNewSalary );
        char          *GetName() const
                        { return Name; }
        o_u4b         GetSalary() const
                        { return Salary; }
};


class PManager : public PEmployee {
    private:
        Link<PDepartment> Manages;
        o_u4b             Bonus;
    public:
        PManager( char *aName, PDepartment *aDep, o_u4b aSalary, o_u4b aBonus );
        virtual o_u4b             hash() const;
        virtual o_4b              compare( const PVirtual & ) const;
        void                      SetBonus( o_u4b aBonus );
        o_u4b                     GetBonus() const
                                    { return Bonus; }
        PDepartment               *GetManagedDepartment() const
                                    { return Manages; }
        static LinkVstr<PManager> FindManager( char *aName );
};


class PDepartment : public PObject {
    private:
        PString          DepName;
        VVSet<PEmployee> SetOfEmployees;
    public:
        PDepartment( char *aName );
        char                         *GetName() const
                                       { return DepName; }
        void                         AddEmployee( PEmployee &anEmployee );
        void                         PrintEmployees( ostream &s );
        static LinkVstr<PDepartment> FindDepartment( char *aName );
};


//***** Implementation der Klasse PEmployee

Implement PClassObj<PEmployee>;  // Deklaration des Klassenobjekts

PEmployee::PEmployee( char *aName, o_u4b aSalary )
    : PVirtual(), Name( aName ), Salary( aSalary )   {}

o_u4b PEmployee::hash() const
{
    return Salary;
}

o_4b PEmployee::compare( const PVirtual &other ) const
{
    o_4b c = VElemental::compare( is_a(), other.is_a() );
    return c != 0 ? c : Name == AS( PEmployee, &other )->GetName() &&
                        Salary == AS( PEmployee, &other )->GetSalary();
}

void PEmployee::SetName( char *aNewName )
{
    dirty();
    Name = aNewName;
}

void PEmployee::SetSalary( o_u4b aNewSalary )
{
    dirty();
    Salary = aNewSalary;
}

//***** Implementation der Klasse PManager

Implement PClassObj<PManager>;  // Deklaration des Klassenobjekts

PManager::PManager( char *aName, PDepartment *aDep, o_u4b aSalary, o_u4b aBonus )
    : PEmployee( aName, aSalary ), Manages( aDep ), Bonus( aBonus )   {}

o_u4b PManager::hash() const
{
    return Salary;
}

o_4b PManager::compare( const PVirtual &other ) const
{
    o_4b c = PEmployee::compare( other );
    return c != 0 ? c : Bonus == AS( PManager, &other )->GetBonus();
}

void PManager::SetBonus( o_u4b aBonus )
{
    dirty();
    Bonus = aBonus;
}

LinkVstr<PManager> PManager::FindManager( char *aName )
{
    return PClassObject<PManager>::Object().select(
        NULL,
        FALSE,
        PPredicate( PAttribute( "PEmployee::Name" ) == aName ) );
}

//***** Implementation der Klasse PDepartment

Implement PClassObj<PDepartment>;  // Deklaration des Klassenobjekts
Implement PClassObj<VVSet<PEmployee>>;

PDepartment::PDepartment( char *aName )
    : DepName( aName )  {}

void PDepartment::AddEmployee( PEmployee &anEmployee )
{
    dirty();
    SetOfEmployees.add( anEmployee );
}

void PDepartment::PrintEmployees( ostream &s )
{
    VVSetIterator<PEmployee> iterator( SetOfEmployees );

    s << "List of Employees:\n";
    for( int i = 0; i < SetOfEmployees.size(); i++ ) {
        s << iterator->GetName() << "\n";
        iterator++;
    }
}

LinkVstr<PDepartment> PDepartment::FindDepartment( char *aName )
{
    return PClassObject<PDepartment>::Object().select(
        NULL,
        FALSE,
        PPredicate( PAttribute( "PDepartment::DepName" ) == aName ) );
}

//***** HAUPTPROGRAMM *****

#ifndef GEN_VT

int main( int argc, char *argv[] )
{
    LinkVstr<PDepartment> DepSet;
    LinkVstr<PManager>    ManSet;
    PEmployee             *Emp;
    char                  command;
    char                  Name[ 80 ];
    char                  DepName[ 80 ];
    int                   Salary;
    int                   Bonus;

    ::dom = new PDOM;
    ::dom->beginsession( "companydb", NULL );

    cout << "Command (M)anager, (D)epartment, (E)mployee, (L)ist: ";
    cin >> &command;   // Kommando einlesen
    switch( command ) {
        case 'm':   // Neuen Manager erzeugen
            cout << "Manager-Name: ";        cin >> Name;
            cout << "Manager-Salary: ";      cin >> Salary;
            cout << "Manager-Bonus: ";       cin >> Bonus;
            cout << "Manages Department: ";  cin >> DepName;
            DepSet = PDepartment::FindDepartment( DepName );
            if( DepSet.is_empty() ) {
                cout << "Department not found!\n";
                break;
            }
            Emp = new Persistent PManager( Name, DepSet[ 0 ], Salary, Bonus );
            DepSet[ 0 ]->AddEmployee( *Emp );
            break;
        case 'd':   // Neues Department erzeugen
            cout << "Department-Name: ";  cin >> Name;
            new Persistent PDepartment( Name );
            break;
        case 'e':   // Neuen Employee erzeugen
            cout << "Employee-Name: ";          cin >> Name;
            cout << "Employee-Salary: ";        cin >> Salary;
            cout << "Belongs to Department: ";  cin >> DepName;
            DepSet = PDepartment::FindDepartment( DepName );
            if( DepSet.is_empty() ) {
                cout << "Department not found!\n";
                break;
            }
            Emp = new Persistent PEmployee( Name, Salary );
            DepSet[ 0 ]->AddEmployee( *Emp );
            break;
        case 'l':   // Alle Employees eines Managers ausgeben
            cout << "Manager-Name: ";  cin >> Name;
            ManSet = PManager::FindManager( Name );
            if( ManSet.is_empty() ) {
                cout << "Manager not found!\n";
                break;
            }
            cout << "Deparment-Name: " <<
                  ManSet[ 0 ]->GetManagedDepartment()->GetName() << "\n";
            ManSet[ 0 ]->GetManagedDepartment()->PrintEmployees( cout );
            break;
        default:
            cout << "Illegal command\n";
            break;
    }
    ::dom->endsession();
    ::dom->exit( 0 );
}

#endif


!!! Listing "Suchausdrcke in Versant..."

PPredicate pred = 
	PAttribute ("PEmployee::Name") == "Frusty" &&
	PAttribute ("PManager::Bonus") <= 123;

!!! Ende Listing "Suchausdrcke in Versant..."

