

Shell-Programmierung

praxis

shell-semaphor.lis

!!! Abbildungen

:tb

!!! Abb. 1

@u^vers = schein-sem1.sh

....
if test -f semfile
then
        # Semaphore belegt, also zum Beispiel warten
        ....
else
        echo > semfile
        # Beginn des kritischen Abschnitts
        ....
        # Ende des kritischen Abschnitts
        rm -f semfile
fi
....

!!! Ende Abb. 1

!!! Abb. 2

@u^vers = schein-sem2.sh

....
if (umask 222; echo > semfile)
then
        # OK -> Datei gab es noch nicht
        # Beginn des kritischen Abschnitts
        ....
        # Ende des kritischen Abschnitts
        rm semfile
else
        # Fehler -> Datei existiert bereits
        # Semaphore belegt, also zum Beispiel warten
        ....
fi
....

!!! Ende Abb. 2

!!! Abb. 3

@u^vers = fifo-daemon1

#!/bin/sh
mkfifo semfifo
while :
do      sleep 3600
done < semfifo &
exec >> semfifo
echo
wait

!!! Ende Abb. 3

!!! Abb. 4

@u^vers = fifo-client.sh

read dummy < semfifo
# Anfang des kritischen Abschnitts
....
# Ende des kritischen Abschnitts
echo >> semfifo

!!! Ende Abb. 4

!!! Abb. 5

@u^vers = demo.sh

#
# Demo-Programm fuer Shell-Semaphoren
#
PROG=`basename $0`
USAGE="$PROG -{file|fifo} taskdir"

#
# Aufrufargumente pruefen
#
case $# in
2);; *) >&2 echo "Usage: $USAGE"; exit 1
esac

#
# File oder Fifo Semaphoren inkludieren
#
. sem$1.sh
shift

taskdir=$1;
shift
test -d "$taskdir" || {
        >&2 echo "$PROG: keine Taetigkeitsbeschreibung: $taskdir"
        exit 2
}

set x `find "$taskdir" -type f -print | sed 's/[^\/]*\///' | sort`
shift

#
# Meldungsausgabe mit Zeitstempel
#
message() {
        echo "(`date +%H:%M:%S`)        $name:  $*"
}

#
# Betriebsmittel anfordern und auf Zuteilung warten
#
request() {
        # Belegungswunsch anmelden
        message "brauche $resource ..."
        lock $resource
        # Belegungswunsch akzeptiert
        message "       habe $resource ..."
}

#
# Betriebsmittel freigeben
#
release() {
        # Freigabe
        message "               fertig mit $resource"
        unlock $resource
}

#
# Konkurrierende Prozesse starten
#
pids=
for name
do
        test -f $taskdir/$name && (
                while
                        read usetime resource
                do
                        test $resource &&
                                request $resource
                        sleep $usetime
                        test $resource &&
                                release $resource
                done < $taskdir/$name
                exit 0
        ) & pids="$pids $!"
done

trap 'kill $pids' 1 2 3
wait

!!!! Ende Abb. 5

!!! Abb. 6

@u^vers = sem-file.sh

#
# Semaphoren auf Basis von Lock-Files
#

rm -f *.lck
: > lockbase
trap '
        rm -f lockbase
        exit 129
' 0 1 2 3

lock() {
        until
                ln lockbase $1.lck 2>/dev/null
        do
                case $sleeptime in 0) continue; esac
                sleep ${sleeptime:-1}
        done
}

unlock() {
        rm -f $1.lck
}

!!! Ende Abb. 6

!!!! Abb. 7

@u^vers = sem-fifo.sh

#
# Semaphoren auf Basis von Named Pipes
#

lock() {
        read dummy <$1.lck 
}

unlock() {
        echo >>$1.lck
}

!!! Ende Abb. 7

!!! Abb. 8

@u^vers = daemon.sh

#
# Manager-Prozess fuer FIFO-Semaphoren
#
procs=
files=
for parm
do      (
                rm -f $parm.lck
                /etc/mknod $parm.lck p || exit
                while 
                        sleep 3600 &
                        trap 'exec kill $!' 1 2 3
                        wait
                do :
                done < $parm.lck &
                trap 'exec kill -1 $!' 1 2 3
                exec >> $parm.lck
                echo
                wait
        ) &
        procs="$procs $!"
        files="$files $parm.lck"
done
trap '
        test -n "$procs" && kill -1 $procs
        procs=
        test -n "$files" && rm -f $files
        files=
        exit 129
' 0 1 2 3
wait

!!! Ende Abb. 8
